"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const scrollIntoView = require('scroll-into-view');
class Actor {
    constructor(rootBuilder, options) {
        this._builder = rootBuilder;
        this.isNull = true;
        this._options = options;
    }
    get root() {
        return this._root;
    }
    get options() {
        return this._options;
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                this._root = yield this._builder();
                if (this._root !== null)
                    this.name = `${this._root.tagName}-${this._root.id}-${this._root.className}`.toLocaleLowerCase();
                this.isNull = this._root == null;
                resolve();
            }));
        });
    }
    getInformation() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                yield this.build();
                if (this.isNull) {
                    resolve({
                        borderRadius: {
                            topRight: "50px",
                            topLeft: "50px",
                            bottomRight: "50px",
                            bottomLeft: "50px"
                        },
                        position: {
                            width: 0,
                            height: 0,
                            x: window.innerWidth / 2,
                            y: window.innerHeight / 2
                        }
                    });
                }
                else {
                    const rect = this._root.getBoundingClientRect();
                    const style = getComputedStyle(this._root);
                    resolve({
                        borderRadius: {
                            topLeft: style.borderTopLeftRadius,
                            topRight: style.borderTopRightRadius,
                            bottomLeft: style.borderBottomLeftRadius,
                            bottomRight: style.borderBottomRightRadius
                        },
                        position: {
                            x: rect.x - 2 + window.scrollX,
                            y: rect.y + window.scrollY,
                            width: rect.width - 2,
                            height: rect.height
                        }
                    });
                }
            }));
        });
    }
    _isOutOfViewport() {
        const rect = this._root.getBoundingClientRect();
        // checking if all sides are in
        const out = {
            top: rect.top < 0,
            left: rect.left < 0,
            bottom: rect.bottom > (window.innerHeight || document.documentElement.clientHeight),
            right: rect.right > (window.innerWidth || document.documentElement.clientWidth)
        };
        return out.top || out.left || out.bottom || out.right;
    }
    // https://www.npmjs.com/package/scroll-into-view
    scrollIntoView(options) {
        if (this._isOutOfViewport() || options.centerActorOnFocus) {
            scrollIntoView(this.root, {
                time: options.delay
            });
        }
    }
    toString() {
        return `Actor[${this.name}]`;
    }
}
exports.default = Actor;
