"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const SignPosition_1 = require("./SignPosition");
const SignBackground_1 = require("./SignBackground");
class Sign {
    constructor(options, previousCallback, nextCallback, resetCallback) {
        this.options = options;
        this.background = new SignBackground_1.SignBackground(options.preventContentClicking, options.onBackgroundClick);
        this.root = document.createElement('DIV');
        this.root.classList.add('sign', 'hide-sign');
        document.body.appendChild(this.root);
        this._previousCallback = previousCallback;
        this._nextCallback = nextCallback;
        this._resetCallback = resetCallback;
    }
    isVisible() {
        return !this.root.classList.contains('hide-sign');
    }
    hide(deactivateBackground = false) {
        this.root.classList.add('hide-sign');
        this.root.classList.remove('sign-transition');
        if (deactivateBackground)
            this.background.hide();
    }
    show() {
        this.root.classList.remove('hide-sign');
        this.background.show();
    }
    _getOptions(actor) {
        const options = actor.options;
        let nOptions = Object.assign({}, this.options);
        Object.assign(nOptions, options);
        return nOptions;
    }
    setContent(actor) {
        const nOptions = this._getOptions(actor);
        if (nOptions.signHTML.length !== 0) {
            this.root.innerHTML = nOptions.signHTML;
        }
        else {
            this.root.innerHTML = `<h4>${nOptions.signTitle}</h4><p>${nOptions.signText}</p>`;
        }
        if (nOptions.showControls) {
            this._buildControls();
        }
    }
    setHTMLMessage(actor) {
        return __awaiter(this, void 0, void 0, function* () {
            const information = yield actor.getInformation();
            const nOptions = this._getOptions(actor);
            if (!this.isVisible() && nOptions.showSign) {
                this._setPosition(nOptions.signPosition, information);
                this.show();
            }
        });
    }
    forcePositionRecomputing(actor) {
        return __awaiter(this, void 0, void 0, function* () {
            this.root.classList.add('sign-transition');
            const information = yield actor.getInformation();
            const nOptions = this._getOptions(actor);
            this._setPosition(nOptions.signPosition, information);
        });
    }
    _centerSign() {
        this.root.style.left = "";
        this.root.style.top = "";
        this.root.classList.add('sign-center');
    }
    _setPosition(position, information) {
        this.root.classList.remove('sign-right', 'sign-left', 'sign-top', 'sign-bottom', 'sign-center');
        if (information.position.width === 0 && information.position.height === 0) {
            this._centerSign();
            return;
        }
        const style = getComputedStyle(this.root);
        let height = +style.height.substr(0, style.height.length - 2);
        let width = +style.width.substr(0, style.width.length - 2);
        switch (position) {
            case SignPosition_1.SignPosition.RIGHT:
                this.root.style.left = information.position.x + information.position.width + 20 + 'px';
                this.root.style.top = information.position.y + information.position.height / 2 - +height / 2 + 'px';
                this.root.classList.add('sign-right');
                break;
            case SignPosition_1.SignPosition.LEFT:
                this.root.style.left = +(information.position.x - width) - 2 * 20 + 'px'; // TODO fix
                this.root.style.top = information.position.y + information.position.height / 2 - +height / 2 + 'px';
                this.root.classList.add('sign-left');
                break;
            case SignPosition_1.SignPosition.TOP:
                this.root.style.left = +(information.position.x + information.position.width / 2 - width / 2) + 'px';
                this.root.style.top = information.position.y - height - 20 + 'px';
                this.root.classList.add('sign-top');
                break;
            case SignPosition_1.SignPosition.BOTTOM:
                this.root.style.left = +(information.position.x + information.position.width / 2 - width / 2) + 'px';
                this.root.style.top = information.position.y + information.position.height + 20 + 'px';
                this.root.classList.add('sign-bottom');
                break;
            default:
                throw new Error('not implemented');
        }
    }
    _buildControls() {
        const controlsContainer = document.createElement('div');
        controlsContainer.classList.add('sign-controls');
        const previousBtn = document.createElement('button');
        previousBtn.textContent = "<";
        previousBtn.onclick = () => this._previousCallback();
        const nextBtn = document.createElement('button');
        nextBtn.textContent = ">";
        nextBtn.onclick = () => this._nextCallback();
        const resetBtn = document.createElement('button');
        resetBtn.textContent = "X";
        resetBtn.onclick = () => this._resetCallback();
        controlsContainer.append(previousBtn, nextBtn, resetBtn);
        this.root.appendChild(controlsContainer);
    }
}
exports.Sign = Sign;
