"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SignPosition_1 = require("../../sign/SignPosition");
class OptionBuilder {
    // default values
    static buildOptions(settings) {
        const options = {
            allowHiddenFocus: false,
            autoScroll: true,
            centerActorOnFocus: true,
            delay: 1000,
            loops: false,
            matchRadius: true,
            onBackgroundClick: (spotlight) => spotlight.toggleSpot(),
            onEnd: () => console.log("tour ended"),
            onFocus: (newActor, oldActor) => { console.log(`switching focus from ${oldActor} to ${newActor}`); },
            onFocused: (actor) => { console.log(`now focusing ${actor}`); },
            onLightsOff: () => { },
            preventContentClicking: true,
            preventFocusSpam: true,
            resetOnEnd: true,
            resetOnLightsOff: false,
            shadowColor: '#000000cc',
            showSign: true,
            showControls: true,
            signHTML: '',
            signPosition: SignPosition_1.SignPosition.RIGHT,
            signText: 'Hello there!',
            signTitle: 'Title'
        };
        Object.assign(options, settings);
        OptionBuilder.checkOptions(options);
        return options;
    }
    static checkOptions(options) {
        // checking booleans
        const booleans = [
            'allowHiddenFocus', 'autoScroll', 'centerActorOnFocus', 'loops', 'matchRadius', 'resetOnEnd',
            'resetOnLightsOff', 'showControls'
        ];
        for (const bool of booleans) {
            const value = options[bool];
            if (![true, false].includes(value)) {
                console.error(`"${bool}" option has wrong value, should equal true or false.`);
            }
        }
        // checking numbers
        const numbers = ['delay'];
        for (const number of numbers) {
            const value = options[number];
            if (typeof value !== 'number' || Number.isNaN(value)) {
                console.error(`"${number}" option has wrong value, should be a number.`);
            }
        }
        // checking colors
        const colors = ['shadowColor'];
        // https://gist.github.com/olmokramer/82ccce673f86db7cda5e#gistcomment-2174868
        const regex = /^(#[0-9a-f]{3}|#(?:[0-9a-f]{2}){2,4}|(rgb|hsl)a?\((-?\d+%?[,\s]+){2,3}\s*[\d\.]+%?\))$/;
        for (const color of colors) {
            const value = options[color];
            if (!value.match(regex)) {
                console.error(`"${color}" is not correctly formatted.`);
            }
        }
    }
}
exports.OptionBuilder = OptionBuilder;
